@@ -50,8 +50,8 @@ def add_args(parser):
                              "YAML/JSON for Kolla Ansible")
     parser.add_argument("-ki", "--kolla-inventory", metavar="INVENTORY",
                         help="specify inventory host path "
-                             "(default=$%s/inventory or %s/inventory) for "
-                             "Kolla Ansible" %
+                             "(default=$%s/inventory or %s/inventory) or "
+                             "comma-separated host list for Kolla Ansible" %
                              (CONFIG_PATH_ENV, DEFAULT_CONFIG_PATH))
     parser.add_argument("-kl", "--kolla-limit", metavar="SUBSET",
                         help="further limit selected hosts to an additional "
@@ -87,7 +87,7 @@ def _validate_args(parsed_args, inventory_filename):
         sys.exit(1)
 
     inventory = _get_inventory_path(parsed_args, inventory_filename)
-    result = utils.is_readable_dir(inventory)
+    result = utils.is_readable_dir(parsed_args.kolla_venv)
     if not result["result"]:
         # NOTE(mgoddard): Previously the inventory was a file, now it is a
         # directory to allow us to support inventory host_vars. Support both
@@ -131,8 +131,8 @@ def build_args(parsed_args, command, inventory_filename, extra_vars=None,
             extra_var_value = utils.quote_and_escape(extra_var_value)
             cmd += ["-e", "%s=%s" % (extra_var_name, extra_var_value)]
     if parsed_args.kolla_limit or limit:
-        limit_arg = utils.intersect_limits(parsed_args.kolla_limit, limit)
-        cmd += ["--limit", utils.quote_and_escape(limit_arg)]
+        limits = [l for l in [parsed_args.kolla_limit, limit] if l]
+        cmd += ["--limit", ":&".join(limits)]
     if parsed_args.kolla_skip_tags:
         cmd += ["--skip-tags", parsed_args.kolla_skip_tags]
     if parsed_args.kolla_tags or tags:
@@ -143,6 +143,23 @@ def build_args(parsed_args, command, inventory_filename, extra_vars=None,
     return cmd
 
 
+def _get_environment(parsed_args):
+    """Return an environment dict for executing Kolla Ansible."""
+    env = os.environ.copy()
+    vault.update_environment(parsed_args, env)
+    # If a custom Ansible configuration file exists, use it. Allow
+    # etc/kayobe/kolla/ansible.cfg or etc/kayobe/ansible.cfg.
+    ansible_cfg_path = os.path.join(parsed_args.config_path, "kolla",
+                                    "ansible.cfg")
+    if utils.is_readable_file(ansible_cfg_path)["result"]:
+        env.setdefault("ANSIBLE_CONFIG", ansible_cfg_path)
+    else:
+        ansible_cfg_path = os.path.join(parsed_args.config_path, "ansible.cfg")
+        if utils.is_readable_file(ansible_cfg_path)["result"]:
+            env.setdefault("ANSIBLE_CONFIG", ansible_cfg_path)
+    return env
+
+
 def run(parsed_args, command, inventory_filename, extra_vars=None,
         tags=None, quiet=False, verbose_level=None, extra_args=None,
         limit=None):
@@ -154,8 +171,7 @@ def run(parsed_args, command, inventory_filename, extra_vars=None,
                      verbose_level=verbose_level,
                      extra_args=extra_args,
                      limit=limit)
-    env = os.environ.copy()
-    vault.update_environment(parsed_args, env)
+    env = _get_environment(parsed_args)
     try:
         utils.run_command(" ".join(cmd), quiet=quiet, shell=True, env=env)
     except subprocess.CalledProcessError as e:
